/*** EQUACOES.C ***/

/*

   Neste arquivo devem ser definidas as equacoes das superficies F e G e
tambem os seus dominios. E' utilizado pelos programas INTER.C, GRAF.C  e
2SUP.C.

   Para usar um dos 22 exemplos pre'-definidos abaixo, basta  definir um
simbolo do tipo EXEMPLOn, com n = 1, 2, ..., 21.  Isto pode ser feito na
hora da compilacao (por ex., TCC -DEXEMPLO1 ...  ou gcc -DEXEMPLO1 ... )
ou  acrescentando-se  no  inicio do programa  um  #define ...  (por ex.,
#define EXEMPLO1 )

*/

/* --------------------------------------------------------------------- */

#ifdef EXEMPLO1                                         /* Cilindro/Toro */

double f1(double u, double v) { return 1 + cos(u); }
double f2(double u, double v) { return sin(u); }
double f3(double u, double v) { return v; }

double g1(double u, double v) { return cos(u)*(1 + cos(v)); }
double g2(double u, double v) { return sin(u)*(1 + cos(v)); }
double g3(double u, double v) { return sin(v); }

retangulo Ret1 = {0, DOIS_PI, -PI_SOBRE_2, PI_SOBRE_2},
	  Ret2 = {0, DOIS_PI, 0, DOIS_PI};

#endif

/* --------------------------------------------------------------------- */

#ifdef EXEMPLO2                                         /* Cone/Cilindro */

double f1(double u, double v) { return v; }
double f2(double u, double v) { return sqrt(3)*cos(u)*v; }
double f3(double u, double v) { return sqrt(3)*sin(u)*v; }

double g1(double u, double v) { return 1 + cos(u); }
double g2(double u, double v) { return sin(u); }
double g3(double u, double v) { return v; }

retangulo Ret1 = {-PI, PI, -2, 2}, Ret2 = {-PI, PI, -3.5, 3.5};

#endif

/* --------------------------------------------------------------------- */

#ifdef EXEMPLO3                                             /* Cone/Toro */

double f1(double u, double v) { return v; }
double f2(double u, double v) { return sqrt(3)*cos(u)*v; }
double f3(double u, double v) { return sqrt(3)*sin(u)*v; }

double g1(double u, double v) { return cos(u)*(1 + cos(v)); }
double g2(double u, double v) { return sin(u)*(1 + cos(v)); }
double g3(double u, double v) { return sin(v); }

retangulo Ret1 = {-PI, PI, -2, 2}, Ret2 = {-PI, PI, -PI, PI};

#endif

/* --------------------------------------------------------------------- */

#ifdef EXEMPLO4                                       /* Cilindro/Esfera */

double f1(double u, double v) { return cos(u); }
double f2(double u, double v) { return sin(u) - 1; }
double f3(double u, double v) { return v; }

double g1(double u, double v) { return 2*cos(u)*cos(v); }
double g2(double u, double v) { return 2*cos(u)*sin(v); }
double g3(double u, double v) { return 2*sin(u); }

retangulo Ret1 = {-PI, PI, -3, 3}, Ret2 = {0, DOIS_PI, 0, PI}; /*{-PI, PI, 0, PI};*/

#endif

/* --------------------------------------------------------------------- */

#ifdef EXEMPLO5                       /* Paraboloide/"Seno de revolucao" */

double f1(double u, double v) { return u; }
double f2(double u, double v) { return v; }
double f3(double u, double v) { return  2 - u*u - v*v;}

double g1(double u, double v) { return u; }
double g2(double u, double v) { return sin(u*u + v*v); }
double g3(double u, double v) { return v; }

retangulo Ret1 = {-2.8, 2.8, -2.8, 2.8}, Ret2 = {-4, 4, -4, 4};

#endif

/* --------------------------------------------------------------------- */

#ifdef EXEMPLO6                                     /* Cilindro/Cilindro */

double f1(double u, double v) { return cos(u); }
double f2(double u, double v) { return sin(u); }
double f3(double u, double v) { return v; }

double g1(double u, double v) { return v; }
double g2(double u, double v) { return cos(u); }
double g3(double u, double v) { return sin(u); }

retangulo Ret1 = {-PI, PI, -2, 2}, Ret2 = {-PI, PI, -2, 2};

#endif

/* --------------------------------------------------------------------- */

#ifdef EXEMPLO7                                             /* Toro/Toro */

double f1(double u, double v) { return (3 + cos(u))*sin(v); }
double f2(double u, double v) { return (3 + cos(u))*cos(v); }
double f3(double u, double v) { return sin(u); }

double g1(double u, double v) { return sin(u); }
double g2(double u, double v) { return (3 + cos(u))*cos(v); }
double g3(double u, double v) { return (3 + cos(u))*sin(v); }

retangulo Ret1 = {-PI, PI, -PI, PI}, Ret2 = {-PI, PI, -PI, PI};

#endif

/* --------------------------------------------------------------------- */

#ifdef EXEMPLO8                                   /* Paraboloide/"Telha" */

double f1(double u, double v) { return u; }
double f2(double u, double v) { return v; }
double f3(double u, double v) { return u*u/3 + v*v/3 - 9; }

double g1(double u, double v) { return u; }
double g2(double u, double v) { return v; }
double g3(double u, double v) { return sin(v) - 3; }

retangulo Ret1 = {-6, 6, -6, 6}, Ret2 = {-8, 8, -8, 8};

#endif

/* --------------------------------------------------------------------- */

#ifdef EXEMPLO9                                            /* Plano/Sela */

double f1(double u, double v) { return u; }
double f2(double u, double v) { return v; }
double f3(double u, double v) { return  1 ;}

double g1(double u, double v) { return v; }
double g2(double u, double v) { return u; }
double g3(double u, double v) { return u*u/10 - v*v/10; }

retangulo Ret1 = {-6.5, 6.5, -6.5, 6.5}, Ret2 = {-6.5, 6.5, -6.5, 6.5};

#endif

/* --------------------------------------------------------------------- */

#ifdef EXEMPLO10                                 /* Paraboloide/Cilindro */

double f1(double u, double v) { return 10 - u*u/4 - v*v/4; }
double f2(double u, double v) { return v; }
double f3(double u, double v) { return u ; }

double g1(double u, double v) { return 3*cos(u); }
double g2(double u, double v) { return 3*sin(u); }
double g3(double u, double v) { return v; }

retangulo Ret1 = {-7.5, 7.5, -7.5, 7.5}, Ret2 = {-PI, PI, -9, 9};

#endif

/* --------------------------------------------------------------------- */

#ifdef EXEMPLO11                                        /* "Mola"/Esfera */

double f1(double u, double v) { return (8 + 2*cos(u))*cos(v); }
double f2(double u, double v) { return (8 + 2*cos(u))*sin(v); }
double f3(double u, double v) { return 2*sin(u) + v ; }

double g1(double u, double v) { return 9*cos(u)*sin(v); }
double g2(double u, double v) { return 9*sin(u) -  8; }
double g3(double u, double v) { return 9*cos(u)*cos(v); }

retangulo Ret1 = {-PI, PI, -9, 11}, Ret2 = {-PI, PI, 0, PI};

#endif

/* --------------------------------------------------------------------- */

#ifdef EXEMPLO12                                 /* Cilindro/Paraboloide */

double f1(double u, double v) { return v + 4*sin(u); }
double f2(double u, double v) { return 1.5*v ; }
double f3(double u, double v) { return 5 + v + 4*cos(u) ; }

double g1(double u, double v) { return u; }
double g2(double u, double v) { return v; }
double g3(double u, double v) { return 9 - u*u/5 - v*v/5; }

retangulo Ret1 = {-PI, PI, -9, 9}, Ret2 = {-7.5, 7.5, -7.5, 7.5};

#endif

/* --------------------------------------------------------------------- */

#ifdef EXEMPLO13                               /* Garrafa de Klein/Plano */

double f1(double u, double v) {
  if (u <= 6.28 && u > 3.14)
    return 6*cos(u)*(1 + sin(u)) + 4*(1 - cos(u)/2)*cos(v + M_PI);
  else
    return 6*cos(u)*(1 + sin(u)) + 4*(1 - cos(u)/2)*cos(v)*cos(u);
}
double f2(double u, double v) {
  if (u <= 6.28 && u > 3.14)
    return 16*sin(u);
  else
    return 16*sin(u) + 4*(1 - cos(u)/2)*sin(u)*cos(v);
}
double f3(double u, double v) { return 4*(1 - cos(u)/2)*sin(v); }

double g1(double u, double v) { return u; }
double g2(double u, double v) { return 1; }
double g3(double u, double v) { return v /* - u/2 - v/2 */ ; }

retangulo Ret1 = {0, DOIS_PI+0.1, 0, 0.1+DOIS_PI}, Ret2 = {-15, 10, -15, 20};

#endif

/* --------------------------------------------------------------------- */

#ifdef EXEMPLO14                                   /* Cilindro/Helicoide */

double f1(double u, double v) { return 3*cos(u); }
double f2(double u, double v) { return 3*sin(u); }
double f3(double u, double v) { return v; }

double g1(double u, double v) { return v*cos(u); }
double g2(double u, double v) { return v*sin(u); }
double g3(double u, double v) { return u; }

retangulo Ret1 = {-PI, PI, -7, 7}, Ret2 = {-7, 7, -7, 7};

#endif

/* --------------------------------------------------------------------- */

#ifdef EXEMPLO15                        /* Faixa de Mobius/Sup. Racional */

double f1(double x, double y) { return (3 - y*sin(x/2))*sin(x); }
double f2(double x, double y) { return (3 - y*sin(x/2))*cos(x); }
double f3(double x, double y) { return y*cos(x/2); }

double g1(double x, double y) { return x + 1.5*x/(x*x + y*y + 1); }
double g2(double x, double y) { return 0.15*(1.0/3.0*y*y*y - x*x*y +
				    3*y - 2.5*y/(x*x + y*y + 1)); }
double g3(double x, double y) { return 0.08*(1.0/3.0*x*x*x - x*y*y -
				    3*y - 2*x/(x*x + y*y + 1)) ; }

retangulo Ret1 = {0, 4*PI, -1, 1}, Ret2 = {-3.5, 3.5, -3.5, 3.5};

#endif

/* --------------------------------------------------------------------- */

#ifdef EXEMPLO16                             /* "Concha"/Sup. Polinomial */

double f1(double u, double v) { return u*cos(v)*sin(u); }
double f2(double u, double v) { return u*cos(u)*cos(v); }
double f3(double u, double v) { return -u*sin(v); }

double g1(double u, double v) { return u; }
double g2(double u, double v) { return v; }
double g3(double u, double v) { return u*u*v/50 - u*v*v/50; }

retangulo Ret1 = {0, 3*PI, 0, 2*PI}, Ret2 = {-8, 8, -8, 8};

#endif

/* --------------------------------------------------------------------- */

#ifdef EXEMPLO17                                 /* "Toro enrolado"/Sela */

double f1(double u, double v) {
  return (4 + (2 + cos(v))*cos(5*u) - sin(v)*sin(5*u))*cos(u); }
double f2(double u, double v) {
  return (4 + (2 + cos(v))*cos(5*u) - sin(v)*sin(5*u))*sin(u) ; }
double f3(double u, double v) {
  return      (2 + cos(v))*sin(5*u) + sin(v)*cos(5*u); }

double g1(double u, double v) { return u; }
double g2(double u, double v) { return v; }
double g3(double u, double v) { return (u*u-v*v)/15.0; }

retangulo Ret1 = {-PI, PI, -PI, PI}, Ret2 = {-6, 6, -6, 6};

#endif

/* --------------------------------------------------------------------- */

#ifdef EXEMPLO18                                        /* Toro/Cilindro */

double f1(double u, double v) { return (10 - 5*sin(u))*sin(v); }
double f2(double u, double v) { return 5*cos(u); }
double f3(double u, double v) { return (10 - 5*sin(u))*cos(v); }

double g1(double u, double v) { return v; }
double g2(double u, double v) { return 5*cos(u); }
double g3(double u, double v) { return 5*sin(u); }

retangulo Ret1 = {0, DOIS_PI, 0, DOIS_PI}, Ret2 = {0, DOIS_PI, -20, 20};

#endif

/* --------------------------------------------------------------------- */

#ifdef EXEMPLO19                          /* Sup. Racional/Sup. Racional */

double f1(double u, double v) { return u; }
double f2(double u, double v) { return v; }
double f3(double u, double v) { return 10*(u*u - v*v)/(2 + u*u*u*u + v*v*v*v); }

double g1(double u, double v) { return u/(v*v+1); }
double g2(double u, double v) { return v; }
double g3(double u, double v) { return (u - 1)/2; }

retangulo Ret1 = {-4, 4, -4, 4}, Ret2 = {-4, 5, -4, 4};

#endif

/* ---------------------------------------------------------------------- */

#ifdef EXEMPLO20                                 /* Faixa de Mobius/Plano */

double f1(double x, double y) { return (3 - y*sin(x/2))*sin(x); }
double f2(double x, double y) { return (3 - y*sin(x/2))*cos(x); }
double f3(double x, double y) { return y*cos(x/2); }

double g1(double x, double y) { return x; }
double g2(double x, double y) { return y; }
double g3(double x, double y) { return  x/10 + y/15; }

retangulo Ret1 = {0, 4*PI, -1, 1}, Ret2 = {-4, 4, -4, 4};

#endif

/* ---------------------------------------------------------------------- */

#ifdef EXEMPLO21                                   /* Catenoide/Catenoide */

double f3(double u, double v) { return (2 + 0.25*cosh(v))*cos(u); }
double f2(double u, double v) { return (2 + 0.25*cosh(v))*sin(u); }
double f1(double u, double v) { return v; }

double g1(double u, double v) { return (2 + 0.25*cosh(v))*cos(u); }
double g2(double u, double v) { return (2 + 0.25*cosh(v))*sin(u); }
double g3(double u, double v) { return v; }

retangulo Ret1 = {0, 2*PI, -3.5, 3.5}, Ret2 = {0, 2*PI, -3.5, 3.5};

#endif

/* ---------------------------------------------------------------------- */

#ifdef EXEMPLO22            /* Sup. revolucao f(x)=2exp(-x)cos(2x)/Plano */

double f1(double u, double v) { return u; }
double f2(double u, double v) { return v; }
double f3(double u, double v) { return 2*exp((-u*u-v*v)/5)*cos(2*(u*u+v*v)); }

double g1(double u, double v) { return u; }
double g2(double u, double v) { return v; }
double g3(double u, double v) { return 0; }

retangulo Ret1 = {-3, 3, -3, 3}, Ret2 = {-3, 3, -3, 3};

#endif

/* ---------------------------------------------------------------------- */

#ifdef EXEMPLO23                       /* Plano/Sup. polinomial de grau 6 */

double f1(double u, double v) { return u; }
double f2(double u, double v) { return v; }
double f3(double u, double v) { return
	      (0.5 - 3*u*u - v*v)*(0.6 - 3*u*u - v*v)*(0.7 - 3*u*u - v*v); }

double g1(double u, double v) { return u; }
double g2(double u, double v) { return v; }
double g3(double u, double v) { return 0; }

retangulo Ret1 = {-1, 1, -1, 1},
	  Ret2 = {-1, 1, -1, 1};
#endif

/* ---------------------------------------------------------------------- */

#ifdef EXEMPLO24                       /* Plano/Sup. polinomial de grau 4 */

double f1(double u, double v) { return u; }
double f2(double u, double v) { return v; }
double f3(double u, double v) { return (0.5 - u*u - v*v)*(0.55 - u*u - v*v); }

double g1(double u, double v) { return u; }
double g2(double u, double v) { return v; }
double g3(double u, double v) { return 0; }

retangulo Ret1 = {-1, 1, -1, 1},
	  Ret2 = {-1, 1, -1, 1};

#endif

/* ---------------------------------------------------------------------- */

#ifdef EXEMPLO25 		            /* Garrafa de Klein/Plano (2) */

double f1(double u, double v) {
  if (u <= 6.28 && u > 3.14)
    return 6*cos(u)*(1 + sin(u)) + 4*(1 - cos(u)/2)*cos(v + M_PI);
  else
    return 6*cos(u)*(1 + sin(u)) + 4*(1 - cos(u)/2)*cos(v)*cos(u);
}
double f2(double u, double v) {
  if (u <= 6.28 && u > 3.14)
    return 16*sin(u);
  else
    return 16*sin(u) + 4*(1 - cos(u)/2)*sin(u)*cos(v);
}
double f3(double u, double v) { return 4*(1 - cos(u)/2)*sin(v); }

double g1(double u, double v) { return u; }
double g2(double u, double v) { return v; }
double g3(double u, double v) { return 1 - v/2 - u/2; }

retangulo Ret1 = {0, DOIS_PI+0.1, 0, 0.1+DOIS_PI},
	  Ret2 = {-20, 15, -20, 25  };

#endif

/* ---------------------------------------------------------------------- */

#ifdef EXEMPLO26                              /* Sup. polinomial/Cilindro */

double f1(double u, double v) { return u; }
double f2(double u, double v) { return v; }
double f3(double u, double v) { return 2 - (u*u*v*v/5 + 5)*(v*v*u*u/5 + 5)/25; }

double g1(double u, double v) { return u; }
double g2(double u, double v) { return cos(v); }
double g3(double u, double v) { return sin(v); }

retangulo Ret1 = {-2, 2, -2, 2},
	  Ret2 = {-2, 2, -PI, PI};

#endif

/* ---------------------------------------------------------------------- */

#ifdef EXEMPLO27                   /* Sup. polinomial grau 4/Poli. grau 3 */

double f1(double u, double v) { return u; }
double f2(double u, double v) { return v; }
double f3(double u, double v) { return (2*u*u*u*u + v*v*v*v)/10.0; }

double g1(double u, double v) { return u; }
double g2(double u, double v) { return v; }
double g3(double u, double v) { return (3*u*u*v - v*v + 2*v*v*v)/10.0; }

retangulo Ret1 = {-1.8, 1.8, -0.3, 2.3},
	  Ret2 = {-1.8, 1.8, -0.3, 2.3};

#endif

/* ---------------------------------------------------------------------- */

#ifdef EXEMPLO28                       /* Plano/Sup. polinomial de grau 6 */

double f1(double u, double v) { return u; }
double f2(double u, double v) { return v; }
double f3(double u, double v) { return
	    (0.5 - 3*u*u + v*v)*(0.6 - 3*u*u + v*v)*(0.7 - 3*u*u + v*v)/10.0; }

double g1(double u, double v) { return u; }
double g2(double u, double v) { return v; }
double g3(double u, double v) { return 0; }

retangulo Ret1 = {-1, 1, -1, 1},
	  Ret2 = {-1, 1, -1, 1};
#endif

/* ---------------------------------------------------------------------- */

/*** FIM DE EQUACOES.C ***/
